@@ -10,31 +10,32 @@ import arjun.core.config as mem
 from arjun.core.bruter import bruter
 from arjun.core.exporter import exporter
 from arjun.core.requester import requester
-from arjun.core.anomaly import define
+from arjun.core.anomaly import define, compare
 from arjun.core.utils import fetch_params, stable_request, random_str, slicer, confirm, populate, reader, nullify, prepare_requests, compatible_path
 
 from arjun.plugins.heuristic import heuristic
 
-arjun_dir = compatible_path(mem.__file__.replace('/core/config.py', ''))
+arjun_dir = compatible_path(mem.__file__.replace(compatible_path('/core/config.py'), ''))
 
 parser = argparse.ArgumentParser() # defines the parser
 # Arguments that can be supplied
-parser.add_argument('-u', help='target url', dest='url')
-parser.add_argument('-o', '-oJ', help='path for json output file', dest='json_file')
-parser.add_argument('-oT', help='path for text output file', dest='text_file')
-parser.add_argument('-oB', help='port for burp suite proxy', dest='burp_port')
-parser.add_argument('-d', help='delay between requests', dest='delay', type=float, default=0)
-parser.add_argument('-t', help='number of threads', dest='threads', type=int, default=2)
-parser.add_argument('-w', help='wordlist path', dest='wordlist', default=arjun_dir+'/db/default.txt')
-parser.add_argument('-m', help='request method: GET/POST/XML/JSON', dest='method', default='GET')
-parser.add_argument('-i', help='import targets from file', dest='import_file', nargs='?', const=True)
-parser.add_argument('-T', help='http request timeout', dest='timeout', type=float, default=15)
-parser.add_argument('-c', help='chunk size/number of parameters to be sent at once', type=int, dest='chunks', default=500)
-parser.add_argument('-q', help='quiet mode, no output', dest='quiet', action='store_true')
-parser.add_argument('--headers', help='add headers', dest='headers', nargs='?', const=True)
-parser.add_argument('--passive', help='collect parameter names from passive sources', dest='passive')
-parser.add_argument('--stable', help='prefer stability over speed', dest='stable', action='store_true')
-parser.add_argument('--include', help='include this data in every request', dest='include', default={})
+parser.add_argument('-u', help='Target URL', dest='url')
+parser.add_argument('-o', '-oJ', help='Path for json output file.', dest='json_file')
+parser.add_argument('-oT', help='Path for text output file.', dest='text_file')
+parser.add_argument('-oB', help='Port for output to Burp Suite Proxy. Default port is 8080.', dest='burp_port', nargs='?', const=8080)
+parser.add_argument('-d', help='Delay between requests in seconds. (default: 0)', dest='delay', type=float, default=0)
+parser.add_argument('-t', help='Number of concurrent threads. (default: 5)', dest='threads', type=int, default=5)
+parser.add_argument('-w', help='Wordlist file path. (default: {arjundir}/db/large.txt)', dest='wordlist', default=arjun_dir+'/db/large.txt')
+parser.add_argument('-m', help='Request method to use: GET/POST/XML/JSON. (default: GET)', dest='method', default='GET')
+parser.add_argument('-i', help='Import target URLs from file.', dest='import_file', nargs='?', const=True)
+parser.add_argument('-T', help='HTTP request timeout in seconds. (default: 15)', dest='timeout', type=float, default=15)
+parser.add_argument('-c', help='Chunk size. The number of parameters to be sent at once', type=int, dest='chunks', default=500)
+parser.add_argument('-q', help='Quiet mode. No output.', dest='quiet', action='store_true')
+parser.add_argument('--headers', help='Add headers. Separate multiple headers with a new line.', dest='headers', nargs='?', const=True)
+parser.add_argument('--passive', help='Collect parameter names from passive sources like wayback, commoncrawl and otx.', dest='passive', nargs='?', const='-')
+parser.add_argument('--stable', help='Prefer stability over speed.', dest='stable', action='store_true')
+parser.add_argument('--include', help='Include this data in every request.', dest='include', default={})
+parser.add_argument('--disable-redirects', help='disable redirects', dest='disable_redirects', action='store_true')
 args = parser.parse_args() # arguments to be parsed
 
 if args.quiet:
@@ -58,6 +59,8 @@ mem.var['method'] = mem.var['method'].upper()
 
 if mem.var['stable'] or mem.var['delay']:
     mem.var['threads'] = 1
+if mem.var['wordlist'] in ('large', 'medium', 'small'):
+    mem.var['wordlist'] = f'{arjun_dir}/db/{mem.var["wordlist"]}.txt'
 
 try:
     wordlist_file = arjun_dir + '/db/small.txt' if args.wordlist == 'small' else args.wordlist
@@ -87,18 +90,19 @@ def narrower(request, factors, param_groups):
     takes a list of parameters and narrows it down to parameters that cause anomalies
     returns list
     """
-    anamolous_params = []
+    anomalous_params = []
     threadpool = ThreadPoolExecutor(max_workers=mem.var['threads'])
     futures = (threadpool.submit(bruter, request, factors, params) for params in param_groups)
     for i, result in enumerate(as_completed(futures)):
         if result.result():
-            anamolous_params.extend(slicer(result.result()))
-        if not mem.var['kill']:
-            print('%s Processing chunks: %i/%-6i' % (info, i + 1, len(param_groups)), end='\r')
-    return anamolous_params
+            anomalous_params.extend(slicer(result.result()))
+        if mem.var['kill']:
+            return anomalous_params
+        print('%s Processing chunks: %i/%-6i' % (info, i + 1, len(param_groups)), end='\r')
+    return anomalous_params
 
 
-def initialize(request, wordlist):
+def initialize(request, wordlist, single_url=False):
     """
     handles parameter finding process for a single request object
     returns 'skipped' (on error), list on success
@@ -108,33 +112,43 @@ def initialize(request, wordlist):
         print('%s %s is not a valid URL' % (bad, url))
         return 'skipped'
     print('%s Probing the target for stability' % run)
-    stable = stable_request(url, request['headers'])
-    if not stable:
+    request['url'] = stable_request(url, request['headers'])
+    if not request['url']:
         return 'skipped'
     else:
         fuzz = random_str(6)
         response_1 = requester(request, {fuzz: fuzz[::-1]})
-        print('%s Analysing HTTP response for anamolies' % run)
+        if single_url:
+            print('%s Analysing HTTP response for anomalies' % run)
         fuzz = random_str(6)
         response_2 = requester(request, {fuzz: fuzz[::-1]})
         if type(response_1) == str or type(response_2) == str:
             return 'skipped'
         factors = define(response_1, response_2, fuzz, fuzz[::-1], wordlist)
-        print('%s Analysing HTTP response for potential parameter names' % run)
+        if single_url:
+            print('%s Analysing HTTP response for potential parameter names' % run)
         found = heuristic(response_1.text, wordlist)
         if found:
             num = len(found)
             s = 's' if num > 1 else ''
             print('%s Heuristic scanner found %i parameter%s: %s' % (good, num, s, ', '.join(found)))
-        print('%s Logicforcing the URL endpoint' % run)
+        if single_url:
+            print('%s Logicforcing the URL endpoint' % run)
         populated = populate(wordlist)
         param_groups = slicer(populated, int(len(wordlist)/mem.var['chunks']))
+        prev_chunk_count = len(param_groups)
         last_params = []
         while True:
             param_groups = narrower(request, factors, param_groups)
+            if len(param_groups) > prev_chunk_count:
+                response_3 = requester(request, {fuzz: fuzz[::-1]})
+                if compare(response_3, factors, [fuzz]) != '':
+                    print('%s Target is misbehaving. Try the --stable swtich.' % bad)
+                    return []
             if mem.var['kill']:
                 return 'skipped'
             param_groups = confirm(param_groups, last_params)
+            prev_chunk_count = len(param_groups)
             if not param_groups:
                 break
         confirmed_params = []
@@ -143,7 +157,7 @@ def initialize(request, wordlist):
             if reason:
                 name = list(param.keys())[0]
                 confirmed_params.append(name)
-                print('%s name: %s, factor: %s' % (res, name, reason))
+                print('%s parameter detected: %s, based on: %s' % (res, name, reason))
         return confirmed_params
 
 
@@ -165,12 +179,17 @@ def main():
                 final_result[url]['params'] = these_params
                 final_result[url]['method'] = request['method']
                 final_result[url]['headers'] = request['headers']
+                exporter(final_result)
+            else:
+                print('%s No parameters were discovered.' % info)
         elif type(request) == list:
             # in case of multiple targets
+            count = 0
             for each in request:
+                count += 1
                 url = each['url']
                 mem.var['kill'] = False
-                print('%s Scanning: %s' % (run, url))
+                print('%s Scanning %d/%d: %s' % (run, count, len(request), url))
                 these_params = initialize(each, list(wordlist))
                 if these_params == 'skipped':
                     print('%s Skipped %s due to errors' % (bad, url))
@@ -179,12 +198,16 @@ def main():
                     final_result[url]['params'] = these_params
                     final_result[url]['method'] = each['method']
                     final_result[url]['headers'] = each['headers']
-                    print('%s Parameters found: %s' % (good, ', '.join(final_result[url])))
+                    exporter(final_result)
+                    print('%s Parameters found: %s\n' % (good, ', '.join(final_result[url]['params'])))
+                    if not mem.var['json_file']:
+                        final_result = {}
+                        continue
+                else:
+                    print('%s No parameters were discovered.\n' % info)
     except KeyboardInterrupt:
         exit()
 
-    exporter(final_result)
-
 
 if __name__ == '__main__':
     main()
