@@ -19,7 +19,7 @@ def burp_export(result):
     """
     proxies = {
         'http': 'http://' + mem.var['burp_port'],
-        'https': 'http://' + mem.var['burp_port']
+        'https': 'https://' + mem.var['burp_port']
     }
     for url, data in result.items():
         if data['method'] == 'GET':
@@ -33,7 +33,7 @@ def text_export(result):
     """
     exports results to a text file, one url per line
     """
-    with open(mem.var['text_file'], 'w+', encoding='utf8') as text_file:
+    with open(mem.var['text_file'], 'a+', encoding='utf8') as text_file:
         for url, data in result.items():
             clean_url = url.lstrip('/')
             if data['method'] == 'JSON':
